@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  *
  * Authors: Xan Lopez
  *          Marco Barisione <marco@barisione.org>
@@ -165,7 +165,7 @@ ft_manager_state_change_reason_to_string (TpFileTransferStateChangeReason reason
       case TP_FILE_TRANSFER_STATE_CHANGE_REASON_NONE:
         return _("No reason was specified");
       case TP_FILE_TRANSFER_STATE_CHANGE_REASON_REQUESTED:
-        return _("The change in state was requested");      
+        return _("The change in state was requested");
       case TP_FILE_TRANSFER_STATE_CHANGE_REASON_LOCAL_STOPPED:
         return _("You canceled the file transfer");
       case TP_FILE_TRANSFER_STATE_CHANGE_REASON_REMOTE_STOPPED:
@@ -200,6 +200,7 @@ ft_manager_update_ft_row (EmpathyFTManager *ft_manager,
   TpFileTransferState state;
   TpFileTransferStateChangeReason reason;
   gboolean incoming;
+  gdouble speed;
 
   row_ref = ft_manager_get_row_from_tp_file (ft_manager, tp_file);
   g_return_if_fail (row_ref != NULL);
@@ -210,6 +211,7 @@ ft_manager_update_ft_row (EmpathyFTManager *ft_manager,
   total_size = empathy_tp_file_get_size (tp_file);
   state = empathy_tp_file_get_state (tp_file, &reason);
   incoming = empathy_tp_file_is_incoming (tp_file);
+  speed = empathy_tp_file_get_speed (tp_file);
 
   switch (state)
     {
@@ -234,6 +236,7 @@ ft_manager_update_ft_row (EmpathyFTManager *ft_manager,
           {
             gchar *total_size_str;
             gchar *transferred_bytes_str;
+            gchar *speed_str;
 
             if (total_size == EMPATHY_TP_FILE_UNKNOWN_SIZE)
               total_size_str = g_strdup (C_("file size", "Unknown"));
@@ -241,17 +244,19 @@ ft_manager_update_ft_row (EmpathyFTManager *ft_manager,
               total_size_str = g_format_size_for_display (total_size);
 
             transferred_bytes_str = g_format_size_for_display (transferred_bytes);
+            speed_str = g_format_size_for_display (speed);
 
             /* translators: first %s is the transferred size, second %s is
              * the total file size */
-            second_line = g_strdup_printf (_("%s of %s"), transferred_bytes_str,
-                total_size_str);
+            second_line = g_strdup_printf (_("%s of %s at %s/s"),
+                transferred_bytes_str, total_size_str, speed_str);
             g_free (transferred_bytes_str);
             g_free (total_size_str);
+            g_free (speed_str);
 
           }
         else
-          second_line = g_strdup (_("Waiting the other participant's response"));
+          second_line = g_strdup (_("Waiting for the other participant's response"));
 
       remaining = empathy_tp_file_get_remaining_time (tp_file);
       break;
@@ -301,8 +306,10 @@ ft_manager_update_ft_row (EmpathyFTManager *ft_manager,
 
   if (remaining < 0)
     {
-      if (state != TP_FILE_TRANSFER_STATE_COMPLETED &&
-          state != TP_FILE_TRANSFER_STATE_CANCELLED)
+      if (state == TP_FILE_TRANSFER_STATE_OPEN)
+        remaining_str = g_strdup (C_("remaining time", "Stalled"));
+      else if (state != TP_FILE_TRANSFER_STATE_COMPLETED &&
+               state != TP_FILE_TRANSFER_STATE_CANCELLED)
         remaining_str = g_strdup (C_("remaining time", "Unknown"));
     }
   else
@@ -334,9 +341,8 @@ ft_manager_update_ft_row (EmpathyFTManager *ft_manager,
 }
 
 static void
-ft_manager_transferred_bytes_changed_cb (EmpathyTpFile *tp_file,
-                                         GParamSpec *pspec,
-                                         EmpathyFTManager *ft_manager)
+ft_manager_refresh_cb (EmpathyTpFile *tp_file,
+                       EmpathyFTManager *ft_manager)
 {
   ft_manager_update_ft_row (ft_manager, tp_file);
 }
@@ -441,7 +447,7 @@ ft_manager_remove_file_from_model (EmpathyFTManager *ft_manager,
   if (update_selection)
     gtk_tree_selection_select_iter (selection, &iter);
 
-  empathy_tp_file_close (tp_file);
+  empathy_tp_file_cancel (tp_file);
 }
 
 static gboolean
@@ -590,7 +596,7 @@ ft_manager_destroy_cb (GtkWidget *widget,
 static void
 ft_manager_build_ui (EmpathyFTManager *ft_manager)
 {
-  GladeXML *glade;
+  GtkBuilder *gui;
   gint x, y, w, h;
   GtkTreeView *view;
   GtkListStore *liststore;
@@ -602,9 +608,8 @@ ft_manager_build_ui (EmpathyFTManager *ft_manager)
   if (ft_manager->priv->window != NULL)
     return;
 
-  filename = empathy_file_lookup ("empathy-ft-manager.glade", "src");
-  glade = empathy_glade_get_file (filename,
-      "ft_manager_dialog", NULL,
+  filename = empathy_file_lookup ("empathy-ft-manager.ui", "src");
+  gui = empathy_builder_get_file (filename,
       "ft_manager_dialog", &ft_manager->priv->window,
       "ft_list", &ft_manager->priv->treeview,
       "open_button", &ft_manager->priv->open_button,
@@ -612,14 +617,14 @@ ft_manager_build_ui (EmpathyFTManager *ft_manager)
       NULL);
   g_free (filename);
 
-  empathy_glade_connect (glade, ft_manager,
+  empathy_builder_connect (gui, ft_manager,
       "ft_manager_dialog", "destroy", ft_manager_destroy_cb,
       "ft_manager_dialog", "response", ft_manager_response_cb,
       "ft_manager_dialog", "delete-event", ft_manager_delete_event_cb,
       "ft_manager_dialog", "configure-event", ft_manager_configure_event_cb,
       NULL);
 
-  g_object_unref (glade);
+  g_object_unref (gui);
 
   /* Window geometry. */
   empathy_geometry_load ("ft-manager", &x, &y, &w, &h);
@@ -633,7 +638,7 @@ ft_manager_build_ui (EmpathyFTManager *ft_manager)
 
   if (w > 0 && h > 0)
     {
-      /* Use the defaults from the glade file if we don't have
+      /* Use the defaults from the ui file if we don't have
        * good w, h geometry. */
       gtk_window_resize (GTK_WINDOW (ft_manager->priv->window), w, h);
     }
@@ -841,8 +846,8 @@ ft_manager_add_tp_file_to_list (EmpathyFTManager *ft_manager,
   ft_manager_update_ft_row (ft_manager, tp_file);
   g_signal_connect (tp_file, "notify::state",
       G_CALLBACK (ft_manager_state_changed_cb), ft_manager);
-  g_signal_connect (tp_file, "notify::transferred-bytes",
-      G_CALLBACK (ft_manager_transferred_bytes_changed_cb), ft_manager);
+  g_signal_connect (tp_file, "refresh",
+      G_CALLBACK (ft_manager_refresh_cb), ft_manager);
 
   gtk_window_present (GTK_WINDOW (ft_manager->priv->window));
 }
@@ -1019,6 +1024,7 @@ ft_manager_display_accept_dialog (EmpathyFTManager *ft_manager,
 
   gtk_message_dialog_format_secondary_text
       (GTK_MESSAGE_DIALOG (dialog),
+       /* Translators: the first %s is the file name, the second %s is the file size */
        _("Do you want to accept the file \"%s\" (%s)?"),
        filename, size_str);
 
